# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): Philippe Merle.
#
# --------------------------------------------------------------------
# $Id: UMLOperations.cs,v 1.3 2004/04/20 13:17:50 carpentier Exp $
# ====================================================================
#
#  This class provides methods to make the constraints writing easier.
#  These methods are defined in the UML Profile for CORBA
#  (formal/02-04-01) section 2.2.3 and in the UML Core Package
#  (formal/03-03-01) section 2.5.3.8 (Additional Operations)
#
# ====================================================================

class UMLOperations
{

    # =================================================================
    # Empty constructor for the UMLOperations.
    # =================================================================
    proc __UMLOperations__(self)
    {
    }


    # =================================================================
    # Sets the UML Package in the UML repository.
    # umlPackage - The UML Package
    # =================================================================
    proc setUMLPackage (self, umlPackage)
    {
        self.umlPackage = umlPackage;
    }


    # =================================================================
    # Gets an array from an IDL sequence.
    # sequence - The IDL sequence
    # =================================================================
    proc sequenceToArray (self, sequence)
    {
        arr = array.create(0);
        for element in sequence {
            arr.append(element);
        }
        return arr;
    }


    # =================================================================
    # Tests if an IDL sequence contains an element.
    # sequence - The IDL sequence
    # element - The element to search
    # =================================================================
    proc sequenceContains (self, sequence, element)
    {
        arr = self.sequenceToArray(sequence);
        return arr.contains(element);
    }


    # =================================================================
    # Gets the opposite end of an association end.
    # element - The element in which the association is linked.
    # end - The near end.
    # =================================================================
    proc otherAssociationEnd (self, element, end)
    {
        for asso in self.associations(element) {
            connection = asso.getConnection();
            if (connection.size() == 2) {
                if (connection.get(0) == end) {
                    return connection.get(1);
                } else {
                    return connection.get(0);
                }
            }
        }
        return end;
    }


    # =================================================================
    # The operation allFeatures results in a Set containing all Features
    # of the Classifier itself and all its inherited Features.
    # From UML Specification, section 2.5.3.8, Additional Operations [1]
    # =================================================================
    proc allFeatures (self, classifier)
    {
        features = classifier.getFeature();
        for generalization in classifier.getGeneralization().toArray() {
            for parentFeature in self.allFeatures(generalization.getParent()) {
                if (!self.sequenceContains(features.toArray(), parentFeature)) {
                    features.add(parentFeature);
                }
            }
        }
        return features.toArray();
    }


    # =================================================================
    # The operation allOperations results in a Set containing all
    # Operations of the Classifier itself and all its inherited Operations.
    # From UML Specification, section 2.5.3.8, Additional Operations [2]
    # =================================================================
    proc allOperations (self, classifier)
    {
        allOperations = array.create(0);
        for feature in self.allFeatures(classifier) {
            if (feature._is_a(org.omg.uml.core.Operation))
                allOperations.append(feature);
        }
        return allOperations;
    }


    # =================================================================
    # The operation allAttributes results in a Set containing all
    # Attributes of the Classifier itself and all its inherited Attributes.
    # From UML Specification, section 2.5.3.8, Additional Operations [4]
    # =================================================================
    proc allAttributes (self, classifier)
    {
        allAttributes = array.create(0);
        for feature in self.allFeatures(classifier) {
            if (feature._is_a(org.omg.uml.core.UmlAttribute))
                allAttributes.append(feature);
        }
        return allAttributes;
    }


    # =================================================================
    # The operation associations results in a Set containing
    # all Associations of the Classifier itself.
    # From UML Specification, section 2.5.3.8, Additional Operations [5]
    # =================================================================
    proc associations (self, classifier)
    {
        all_associations = self.umlPackage.getCore().getAssociation().refAllOfType().toArray();
        assolist = array.create(0);
        for asso in all_associations {
            for connect in asso.getConnection().toArray() {
                if (connect.getParticipant() == classifier) {
                    assolist.append(asso);
                }
            }
        }
        return assolist;
    }


    # =================================================================
    # The operation oppositeAssociationEnds results in a set of
    # all AssociationEnds that are opposite to the Classifier.
    # From UML Specification, section 2.5.3.8, Additional Operations [7]
    # =================================================================
    proc oppositeAssociationEnds (self, classifier)
    {
        endslist = array.create(0);
        for asso in self.associations(classifier) {
            for connect in asso.getConnection().toArray() {
                if (connect.getParticipant() != classifier) {
                    endslist.append(connect);
                }
            }
        }
        return endslist;
    }


    # =================================================================
    # The operation allStereotypes results in a Set containing the
    # ModelElement's Stereotype and all Stereotypes inherited by that
    # Stereotype (as opposed to all Stereotypes inherited by the
    # ModelElement).
    # From UML Profile for CORBA, section 2.2.3.2, ModelElement [1]
    # =================================================================
    proc allStereotypes (self, modelElement)
    {
        stereotypes = modelElement.getStereotype().toArray();
        for stereotype in modelElement.getStereotype().toArray() {
            for generalization in stereotype.getGeneralization().toArray() {
                for parentStereotype in self.allStereotypes(generalization.getParent()) {
                    if (!self.sequenceContains(stereotypes, parentStereotype))
                        stereotypes.append(parentStereotype);
                }
            }
        }
        return stereotypes;
    }


    # =================================================================
    # The operation isStereotyped determines whether the ModelElement
    # has a Stereotype whose name is equal to the input name.
    # From UML Profile for CORBA, section 2.2.3.2, ModelElement [2]
    # =================================================================
    proc isStereotyped (self, modelElement, stereotypeName)
    {
        for stereotype in modelElement.getStereotype().toArray() {
            if (stereotype.getName() == stereotypeName)
            return true;
        }
        return false;
    }


    # =================================================================
    # The operation isStereokinded determines whether the ModelElement
    # has a Stereotype whose name is equal to the input name or if it
    # has a Stereotype one of whose ancestors' name is equal to the
    # input name.
    # From UML Profile for CORBA, section 2.2.3.2, ModelElement [3]
    # =================================================================
    proc isStereokinded (self, modelElement, stereotypeName)
    {
        for stereotype in self.allStereotypes(modelElement) {
            if (self.isStereotyped(stereotype, stereotypeName))
                return true;
        }
        return false;
    }


    # =================================================================
    # The operation navigableOppositeEnds results in a Set containing
    # all navigable AssociationEnds that are opposite to the Classifier.
    # From UML Profile for CORBA, section 2.2.3.2, Classifier [1]
    # =================================================================
    proc navigableOppositeEnds (self, classifier)
    {
        oppositeEnds = array.create(0);
        for ends in self.oppositeAssociationEnds(classifier) {
            if (ends.isNavigable())
                oppositeEnds.append(ends);
        }
        return oppositeEnds;
    }


    # =================================================================
    # The operation allEnds results in a Set containing all
    # AssociationEnds for which the Classifier is the type.
    # From UML Profile for CORBA, section 2.2.3.2, Classifier [2]
    # =================================================================
    proc allEnds (self, classifier)
    {
        connections = array.create(0);
        for asso in self.associations(classifier) {
            for connect in asso.getConnection().toArray() {
                connections.append(connect);
            }
        }
        return connections;
    }


    # =================================================================
    # The operation nonNavigableNearEnds results in a Set containing all
    # AssociationEnds that are adjacent to the Classifier and that are
    # non-navigable.
    # From UML Profile for CORBA, section 2.2.3.2, Classifier [3]
    # =================================================================
    proc nonNavigableNearEnds (self, classifier)
    {
        ends = array.create(0);
        for end in self.allEnds(classifier) {
            if (end.getParticipant() == classifier && !end.isNavigable())
                ends.append(end);
        }
        return ends;
    }


    # =================================================================
    # The operation navigableEnds results in a Set containing all
    # navigable AssociationEnds
    # From UML Profile for CORBA, section 2.2.3.2, Classifier [4]
    # =================================================================
    proc navigableEnds (self, classifier)
    {
        ends = array.create(0);
        for end in self.allEnds(classifier) {
            if (end.isNavigable())
                ends.append(end);
        }
        return ends;
    }

}
